/**
 ** X for KSplash/ML
 ** Copyright (c) 2002, Flaming Sword Productions
 ** All Rights Reserved
 **
 ** $Id: Icon.cpp,v 1.3 2002/06/03 21:26:35 brian Exp $
 **/
#include	<qpainter.h>
#include	<kpixmapeffect.h>
#include	<qtimer.h>
#include	<qwmatrix.h>
#include	<qlabel.h>
#include	<qwidget.h>
#include	<kdebug.h>
#include	<kpixmap.h>
#include	<kwin.h>
#include	"Icon.h"

/**
 ** Icon::Icon(): Constructor.  We will be passed the _largest_ size
 ** possible for this icon, and will be expected to scale it to all
 ** desired sizes.
 **/
		Icon::Icon( const QPixmap& icon, unsigned int min_size, unsigned int max_size, bool optimize )
		: QLabel( 0L, 0L, WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorderEx ),
		  gIcon( icon ),
		  gMinSize( min_size ),
		  gMaxSize( max_size ),
		  gOptimize( optimize ),
		  initialized( false ) {

	resize( 0, 0 );
	KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
	KWin::setType( winId(), NET::Dock );

	_scalePixmapsForCache();

//	setAutoResize( true );
}

/**
 ** Icon::~Icon(): Free up all of our massive quantities of memory.
 **/
		Icon::~Icon() {

	gIcons.clear();
}

/**
 ** Icon::_scalePixmapsForCache(): Prepare pixmaps for each icon size
 ** between 16 and 48 and store them in our pixmap cache.
 **
 ** Note: For fairly obvious reasons, this ought to be an atomic operation.
 **/
void		Icon::_scalePixmapsForCache() {

	/**
	 ** Add a nifty gradient to the background of the icon.  This should make us
	 ** look like early Mac OS X betas.  (Sorry, but the Qt rendering code is 
	 ** NOWHERE near as mature as Aqua yet.  This, of course, is completely 
	 ** understandable!  Apple has spent a fortune getting their eye-candy to 
	 ** where it is.  TrollTech will eventually catch up.)  :)
	 **
	 ** (While we're at it, let's add a slight margin around each icon.  This ought to
	 ** make it look a bit cleaner...  Maybe it will even be configurable eventually...)
	 **/
	KPixmap t( QSize( gIcon.size().width()+4, gIcon.size().height()+4 ) );
	KPixmapEffect::gradient( t, Qt::white, Qt::gray, KPixmapEffect::VerticalGradient );
	QPainter p;
	p.begin(&t);
	p.drawPixmap( 2, 2, gIcon );
	p.end();
	gIcon = t;
	
	/**
	 ** Grumble.  DesktopIcon() failed to load the icon size I requested.  Must correct
	 ** this immediately..
	 **/
	if( gIcon.width() < gMaxSize ) {

		QPixmap tmp( QSize( gMaxSize, gMaxSize ) );
		QWMatrix m;
		float f = (float)tmp.width() / gIcon.width();
		m.scale( f, f );
		tmp = gIcon.xForm( m );
		gIcons.append( new QPixmap( tmp ) );

	} else {
		gIcons.append( new QPixmap( gIcon ) );
	}

	/**
	 ** For each icon size desired, scale the previous icon down by
	 ** one pixel.
	 **/
	QWMatrix matrix;
	QPixmap tmp;
	float xscale;

	for( unsigned int i = gMaxSize - 1; i >= gMinSize; i-- ) {
		xscale = 0.98;

		tmp = QPixmap( QSize( i, i ) );
		matrix.scale( xscale, xscale );
		tmp = gIcons.at( (gMaxSize-i)-1 )->xForm( matrix );
		gIcons.append( new QPixmap( tmp ) );
	}

	initialized = true;
}

/**
 ** Icon::resizeEvent(): Something has resized us.  We need to tell whomever is listening
 ** that we're different now.
 **/
void		Icon::resizeEvent( QResizeEvent *qre ) {

	QLabel::resizeEvent( qre );
	if( initialized ) emit sizeChanged();
}

/**
 ** Icon::_incrementalShow(): Slowly show this icon.  We've already set QLabel::show(),
 ** so all we need to worry about doing here is move()ing and resize()ing.  (And, since
 ** we set QLabel::setAutoResize( true ), we don't even have to do the latter!)
 **/
void		Icon::_incrementalShow() {

	size+=8;
	if( size >= gMaxSize ) {
		setPixmap( *gIcons.at( 0 ) );
		resize( sizeHint() );
		return;
	}

	setPixmap( *gIcons.at( ( gMaxSize - size ) - 1 ) );
	resize( sizeHint() );
	QTimer::singleShot( 100, this, SLOT(_incrementalShow()) );
}

/**
 ** Icon::showEvent(): The icon has been shown.  We will be really fancy
 ** with this - We're going to gradually "zoom" in.  Our IconBar should
 ** automatically adjust our position based on this.
 **/
void		Icon::showEvent( QShowEvent * ) {

	// TRAP: If we haven't been able to scale our icons, we ought to not do
	// anything.
	if( gIcons.isEmpty() ) return;

	size = gMinSize;
	QLabel::show();

	QTimer::singleShot( 10, this, SLOT(_incrementalShow()) );
}
