/**
 ** X for KSplash/ML
 ** Copyright (c) 2002, Flaming Sword Productions
 ** All Rights Reserved
 **
 ** $Id: IconBar.cpp,v 1.2 2002/06/03 18:20:43 brian Exp $
 **/
#include	<kapp.h>
#include	<kwin.h>
#include	<kdebug.h>
#include	<qlabel.h>
#include	<qwidget.h>
#include	<kpixmap.h>
#if ( QT_VERSION >= 300 )
#include	<qdesktopwidget.h>
#else
#include	<kdesktopwidget.h>
#endif
#include	"IconBar.h"

/**
 ** IconBar::IconBar(): Constructor.  Position and size ourselves.
 **/
		IconBar::IconBar() 
		: QLabel( 0L, 0L, WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorderEx ) {

	KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
	KWin::setType( winId(), NET::Dock );
	setPalette( Qt::black );
	resize( 0, 0 );
}

/**
 ** IconBar::~IconBar(): Delete anything we've allocated.
 **/
		IconBar::~IconBar() {

	gIcons.clear();
}

/**
 ** IconBar::appendIcon(): A new icon has been added to our list.
 **/
void		IconBar::appendIcon( Icon *i ) {

	gIcons.append( i );
	connect( i, SIGNAL( sizeChanged() ), this, SLOT( _positionIcons() ) );
	i->show();

	_positionIcons();
}

/**
 ** IconBar::_positionIcons(): Shift and resize the icon objects around
 ** on the screen, based on where and how big they are.
 **
 ** Note: This has to be an atomic operation.  Yech.
 **/
void		IconBar::_positionIcons() {
QRect		desk = _desktopGeometry();

	// TRAP: Nothing to position.
	if( !gIcons.count() ) {
		resize( 0, 0 );
		return;
	}

	int minPosition = 0;
	int maxHeight = 0;
	int totalWidth = 0;
	int accumulatedWidth = 0;

	/**
	 ** Step 1: Determine the width of ALL of our icons.
	 **/
	QValueList< int > x;
	QListIterator<Icon> it( gIcons );
	while( it.current() ) {
		if( it.current()->height() != it.current()->width() ) {
			x.clear();
			return;
		}
		x.append( it.current()->width() );
		totalWidth += it.current()->width();
		++it;
	}

	/**
	 ** For each icon...
	 **/
	for( unsigned int i = 0; i < x.count(); i++ ) {

		/**
		 ** Move the icon to STARTINGPOS + ACCUMULATEDWIDTH.
	 	 **/
		gIcons.at( i )->move( 
			(desk.width() / 2) - (totalWidth/2) + accumulatedWidth,
			desk.height() - gIcons.at( i )->height() - 3
		);

		// Update the ACCUMULATED WIDTH to reflect new information.
		accumulatedWidth += x[i];
	}

	x.clear();

	/**
	 ** Adjust our iconbar to reflect where we are.
	 **/
	resize( totalWidth, 3 );
}

/**
 ** IconBar::resizeEvent(): We've been resized.  Maintain our position
 ** on the screen.  (Don't forget to move our icons!)
 **/
void		IconBar::resizeEvent( QResizeEvent *qre ) {

	QLabel::resizeEvent( qre );
	move( 
		_desktopGeometry().width()/2 - qre->size().width()/2, 
		_desktopGeometry().height() - qre->size().height() 
	);
}

/**
 ** IconBar::_desktopGeometry(): Move common code here to be more concise.
 **/
QRect		IconBar::_desktopGeometry() {

#if ( QT_VERSION > 300 )
	QDesktopWidget *desktop = QApplication::desktop();
	const QRect &screen = desktop->screenGeometry( desktop->primaryScreen() );
#else
	KDesktopWidget *desktop = KApplication::desktop();
	const QRect &screen = desktop->screenGeometry( desktop->primaryScreen() );
#endif

	return( screen );
}
