/**
 ** X for KSplash/ML
 ** Copyright (c) 2002, Flaming Sword Productions
 ** All Rights Reserved
 **
 ** $Id: ThemeX.cpp,v 1.6 2002/06/18 13:12:57 brian Exp $
 **/
#include	<kapp.h>
#include	<kwin.h>
#include	<qlabel.h>
#include	<qwidget.h>
#include	<kstddirs.h>
#include	<kprogress.h>
#include	<kdialogbase.h>
#include	<kiconloader.h>
#if ( QT_VERSION >= 300 )
#include	<qdesktopwidget.h>
#else
#include	<kdesktopwidget.h>
#endif
#include	<ThemeEngine.h>
#include	"Icon.h"
#include	"IconBar.h"
#include	"ThemeX.h"

/**
 ** ThemeX::ThemeX(): Constructor.
 **/
		ThemeX::ThemeX( QWidget *parent, ObjKsTheme *theme )
		:ThemeEngine( parent, theme ) {

	KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
	KWin::setType( winId(), NET::Dock );

	_readSettings();
	_initUi();
}

/**
 ** ThemeX::~ThemeX(): Destructor.
 **/
		ThemeX::~ThemeX() {

	delete gIconBar;
}

/**
 ** ThemeX::slotSetPixmap(): 
 **/
void		ThemeX::slotSetPixmap( const QString& icon ) {

	// Load the icon from the ThemeDir instead of from the system location,
	// if the ThemeDir exists.
	QString s( locate( "data", gTheme->themeDir() + "/" + icon ) );
	if( !s.isNull() ) {
		gIconBar->appendIcon( new Icon( QPixmap( s ), oMinSize, oMaxSize, oOptimizedRendering ) );
		return;
	}

	// Otherwise, just use System functions to load it.
	gIconBar->appendIcon( new Icon( DesktopIcon( icon, oMaxSize ), oMinSize, oMaxSize, oOptimizedRendering ) );
}

/**
 ** ThemeX::_initUi(): 
 **/
void		ThemeX::_initUi() {

	/**
	 ** Prepare our splash screen widget.
	 **/
	QLabel *lbl = new QLabel( this );
	lbl->show();

	QString splashWidget = locate( "data", gTheme->themeDir() + "/Splash.png" );

	/**
	 ** If we can't find THEMEDIR/Splash.png, we ought to just hide ourselves...
	 **/
	if( splashWidget.isNull() ) {
		resize( 0, 0 );
		move( 0, 0 );
	} else {
		QRect desk = _desktopGeometry();

		QPixmap px( splashWidget );
		lbl->setPixmap( px );

		resize( px.width(), px.height() );
		move( desk.width()/2 - px.width()/2, desk.height()/2 - px.height()/2 );
	}

	setBackgroundMode( NoBackground );
	lbl->setBackgroundMode( NoBackground );

	gProgress = new KProgress( lbl );
	gProgress->resize( oProgressPos.size() );
	gProgress->move( oProgressPos.topLeft() );

	/**
	 ** Instantiate our IconBar.
	 **/
	gIconBar = new IconBar;
	gIconBar->show();
}

/**
 ** ThemeX::_desktopGeometry(): 
 **/
QRect		ThemeX::_desktopGeometry() {
#if ( QT_VERSION > 300 )
	QDesktopWidget *desktop = QApplication::desktop();
	const QRect &screen = desktop->screenGeometry( desktop->primaryScreen() );
#else
	KDesktopWidget *desktop = KApplication::desktop();
	const QRect &screen = desktop->screenGeometry( desktop->primaryScreen() );
#endif

	return( screen );
}

/**
 ** ThemeX::readSettings(): Load our configuration from the Theme.rc file.
 **/
void		ThemeX::_readSettings() {

	if( !gTheme ) return;

	// TRAP: Can't operate on a NULL object.
	KConfig *cfg = gTheme->themeConfig();
	if( !cfg ) return;

	if( !cfg->hasGroup( QString("KSplash/ML Theme: %1").arg(gTheme->theme()) ) ) return;
	cfg->setGroup( QString("KSplash/ML Theme: %1").arg(gTheme->theme()) );

	oMinSize = cfg->readNumEntry( "Icon Size Minimum", 16 );
	oMaxSize = cfg->readNumEntry( "Icon Size Maximum", 64 );
	oOptimizedRendering = cfg->readBoolEntry( "Optimized Icon Rendering", true );
	oUseProgress = cfg->readBoolEntry( "Progress Bar Visible", true );
	QRect defaultPos( 30, 250, 375, 20 );
	oProgressPos = cfg->readRectEntry( "Progress Bar Position", &defaultPos );

	// not currently implemented.
	cfg->readBoolEntry( "Icons Jumping", false );
}
